
#pragma ident "@(#)epsf.c	1.3 95/03/31"

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <DPS/dpsXclient.h>


/*
 * calculate pixels per point without querying the server
 * for the default transformation matrix.
 */
double 
pixels_per_point(
    Widget w
)
{
    static const double mm_per_point = (25.4/72.0);

    return ((double)WidthOfScreen(XtScreen(w)) * mm_per_point) 
	/ (double)WidthMMOfScreen(XtScreen(w));
}


static void
status_changed(
	DPSContext ctxt,
	int code
)
{
    switch ( code ) {
    case PSZOMBIE:
	/* Context ran into a PostScript error and died */
	fprintf(stderr, "Encountered a PostScript error, aborting\n");
	XCloseDisplay((Display*) ctxt->priv);
	exit(-1);

    case PSRUNNING:
    case PSNEEDSINPUT:
    case PSFROZEN:
    default:
	break;
    }
}


static Pixmap
pixmap_from_epsf(
	Widget w,
	const char* file_name
)
{
    extern EPSFParaseFileHeader(FILE*, XRectangle*);
    Display* dpy = XtDisplay(w);
    Window xid = XtWindow(w);
    Cardinal depth;
    Dimension width, height;
    DPSContext ctxt;
    Pixmap pix;
    FILE* fp;
    char buf[256];
    XRectangle rect;


    fp = fopen(file_name, "r");
    if ( fp == NULL ) {
	fprintf(stderr, "File %s not found\n", file_name ? file_name : "(none)");
	return None;
    }

    /* Go a-lookin' for the bounding box */
    if ( EPSFParseFileHeader(fp, &rect) == False ) {
	return None;
    }
    rewind(fp);


    /* Calculate width and height in pixels from bounding box 
     * and adjust the size of the window accordingly.
     */
    width  = (unsigned short)((double)rect.width  * pixels_per_point(w));
    height = (unsigned short)((double)rect.height * pixels_per_point(w));
    XtVaSetValues(XtParent(w), XtNwidth, width, XtNheight, height, NULL);

    XtVaGetValues(w, XtNdepth, &depth, NULL);
    pix = XCreatePixmap(dpy, xid, width, height, depth);

    ctxt 
	= XDPSCreateSimpleContext(dpy, pix, 
			DefaultGCOfScreen(XtScreen(w)),
			0, height,
			DPSDefaultTextBackstop,	/* send output to stdout */
			DPSDefaultErrorProc,	/* send errors to stdout */
			(DPSSpace) NULL		/* create a new space */
		);

#ifdef DEBUG
    {
	DPSContext tc
	    = DPSCreateTextContext(
			DPSDefaultTextBackstop, 
			DPSDefaultErrorProc
		);
	DPSChainContext(ctxt, tc);
    }
#endif

    /* libdps requires this when used with Xt */
    XDPSSetEventDelivery(dpy, dps_event_pass_through);

    /* Let us know if there is an error */
    XDPSRegisterStatusProc(ctxt, status_changed);
    XDPSSetStatusMask(ctxt, PSZOMBIEMASK, None, None);
    ctxt->priv = (void*)dpy;		/* used in status proc */


    /* clear page and move starting point according to the bounding box */
    DPSerasepage(ctxt);
    DPStranslate(ctxt, (double)-rect.x, (double)-rect.y);

    /* Now do the EPSF thang */
    PSWBeginEPSF(ctxt);

    while ( fgets(buf, 256, fp) != NULL ) {
	DPSWritePostScript(ctxt, buf, strlen(buf));
    }

    fclose(fp);
    PSWEndEPSF(ctxt);

    /* make sure drawing is finished */
    DPSWaitContext(ctxt);

    return pix;
}



void
main(
	int argc,
	char **argv
)
{
    XtAppContext app;
    Widget topl;
    Widget wid;
    Pixmap pix;
    static const String fallbacks[] = {
	"epsf.title: Simple EPSF Viewer",
	"*.width: 200",		/* arbitrary size, */
	"*.height: 200",	/* will get corrected later */
	NULL
    };

    if ( argc < 2 ) {
	fprintf(stderr, "usage: %s eps-file\n", argv[0]);
	exit(-1);
    }

    topl
	= XtVaAppInitialize(&app, "epsf", NULL, 0, 
			    &argc, argv, (String *)fallbacks, 
			    NULL);
    wid
	= XtVaCreateManagedWidget("wid", widgetClass, topl,
				  NULL);

    XtRealizeWidget(topl);

    /* After realization to get XID of widget */
    pix = pixmap_from_epsf(wid, argv[1]);
    if ( pix == None ) {
	fprintf(stderr, "Creating pixmap for EPSF failed!\n");
	exit(-1);
    }

    /* Let the server worry about expose events.  Note, using
     * a big pixmap can hog memory, be careful in a real app.
     */
    XtVaSetValues(wid, XtNbackgroundPixmap, pix, NULL);

    while ( True ) {
	XEvent event;
	XtAppNextEvent(app, &event);
	if ( !XDPSDispatchEvent(&event) )
	    (void) XtDispatchEvent(&event);
    }
}
/******************************************************************************/
